@@ -191,7 +191,9 @@ static unsigned char std_tables[] = {
 /* misc helper function declarations */
 
 static void dumpbits (bits_t * bits);
+#if 0
 static char *sprintbits (char *str, unsigned int bits, int n);
+#endif
 
 static void huffman_table_load_std_jpeg (JpegDecoder * dec);
 
@@ -317,7 +319,7 @@ generate_code_table (int *huffsize)
   int i;
   int j;
   int k;
-  char str[33];
+  //char str[33];
 
   //int l;
 
@@ -701,7 +703,7 @@ jpeg_decoder_decode_entropy_segment (JpegDecoder * dec, bits_t * bits)
       unsigned char *ptr;
       int component_index;
 
-      JPEG_DEBUG ("%d,%d: component=%d dc_table=%d ac_table=%d",
+      JPEG_LOG ("%d,%d: component=%d dc_table=%d ac_table=%d",
           x, y,
           dec->scan_list[i].component_index,
           dec->scan_list[i].dc_table, dec->scan_list[i].ac_table);
@@ -715,7 +717,7 @@ jpeg_decoder_decode_entropy_segment (JpegDecoder * dec, bits_t * bits)
           dec->dc_huff_table[dc_table_index],
           dec->ac_huff_table[ac_table_index], bits2);
       if (ret < 0) {
-        JPEG_DEBUG ("%d,%d: component=%d dc_table=%d ac_table=%d",
+        JPEG_LOG ("%d,%d: component=%d dc_table=%d ac_table=%d",
             x, y,
             dec->scan_list[i].component_index,
             dec->scan_list[i].dc_table, dec->scan_list[i].ac_table);
@@ -723,7 +725,7 @@ jpeg_decoder_decode_entropy_segment (JpegDecoder * dec, bits_t * bits)
         break;
       }
 
-      JPEG_DEBUG ("using quant table %d", quant_index);
+      JPEG_LOG ("using quant table %d", quant_index);
       oil_mult8x8_s16 (block2, block, dec->quant_table[quant_index],
           sizeof (short) * 8, sizeof(short) * 8, sizeof (short) * 8);
       dec->dc[component_index] += block2[0];
@@ -731,7 +733,7 @@ jpeg_decoder_decode_entropy_segment (JpegDecoder * dec, bits_t * bits)
       oil_unzigzag8x8_s16 (block, sizeof (short) * 8, block2,
           sizeof (short) * 8);
       oil_idct8x8_s16 (block2, sizeof (short) * 8, block, sizeof (short) * 8);
-      oil_trans8x8_u16 (block, sizeof (short) * 8, block2, sizeof (short) * 8);
+      oil_trans8x8_s16 (block, sizeof (short) * 8, block2, sizeof (short) * 8);
 
       ptr = dec->components[component_index].image +
           x * dec->components[component_index].h_oversample +
@@ -801,7 +803,7 @@ jpeg_decoder_addbits (JpegDecoder * dec, unsigned char *data, unsigned int len)
 
   offset = dec->bits.ptr - dec->data;
 
-  dec->data = realloc (dec->data, dec->data_len + len);
+  dec->data = g_realloc (dec->data, dec->data_len + len);
   memcpy (dec->data + dec->data_len, data, len);
   dec->data_len += len;
 
@@ -940,6 +942,7 @@ jpeg_debug (int n, const char *format, ...)
 
 /* misc helper functins */
 
+#if 0
 static char *
 sprintbits (char *str, unsigned int bits, int n)
 {
@@ -954,6 +957,7 @@ sprintbits (char *str, unsigned int bits, int n)
 
   return str;
 }
+#endif
 
 static void
 huffman_table_load_std_jpeg (JpegDecoder * dec)
