@@ -15,12 +15,11 @@
 #define JPEG_LOG(...) \
     JPEG_DEBUG_LEVEL(SWFDEC_LEVEL_LOG, __VA_ARGS__)
 
-#if 1
+#if 0
 #define JPEG_DEBUG_LEVEL(level,...) \
     swfdec_debug_log ((level), __FILE__, __FUNCTION__, __LINE__, __VA_ARGS__)
 #else
-#define JPEG_DEBUG_LEVEL(level,...) \
-    swfdec_debug_log (SWFDEC_LEVEL_LOG, __FILE__, __FUNCTION__, __LINE__, __VA_ARGS__)
+#define JPEG_DEBUG_LEVEL(level,...)
 #endif
 
 
