@@ -6,12 +6,14 @@
 #include "proto.h"
 #include "swfdec.h"
 #include "swfdec_actions.h"
+#include "swfdec_audio.h"
 #include "swfdec_bits.h"
 #include "swfdec_buffer.h"
 #include "swfdec_button.h"
+#include "swfdec_cache.h"
 #include "swfdec_debug.h"
 #include "swfdec_decoder.h"
-#include "swfdec_display.h"
+#include "swfdec_decoder_internal.h"
 #include "swfdec_font.h"
 #include "swfdec_image.h"
 #include "swfdec_layer.h"
