@@ -12,7 +12,11 @@ void swfdec_decoder_sound_buffer_append (SwfdecDecoder * s,
 SwfdecRender *
 swfdec_render_new (void)
 {
-  return g_new0 (SwfdecRender, 1);
+  SwfdecRender *render;
+
+  render = g_new0 (SwfdecRender, 1);
+
+  return render;
 }
 
 void
@@ -33,80 +37,48 @@ swfdec_render_iterate (SwfdecDecoder * s)
 {
   GList *g;
 
+  SWFDEC_DEBUG("iterate, frame_index = %d", s->render->frame_index);
 
-  if (s->render->seek_frame != -1) {
-    SwfdecSound *sound;
+  s->render->frame_index = s->next_frame;
+  s->next_frame = -1;
 
-    s->render->frame_index = s->render->seek_frame;
-    s->render->seek_frame = -1;
+  swfdec_sprite_render_iterate (s, s->main_sprite_seg, s->render);
 
-    sound = SWFDEC_SOUND (s->stream_sound_obj);
-    if (sound)
-      sound->tmpbuflen = 0;
-  } else {
-    if (!s->stopped) {
-      if (s->main_sprite->actions[s->render->frame_index]) {
-        swfdec_action_script_execute (s,
-            s->main_sprite->actions[s->render->frame_index]);
-      }
-    }
-    SWFDEC_DEBUG ("mouse button %d old_mouse_button %d active_button %p",
-        s->mouse_button, s->old_mouse_button, s->render->active_button);
-    if (s->mouse_button && !s->old_mouse_button &&
-        s->render->active_button) {
-      swfdec_button_execute (s, SWFDEC_BUTTON (s->render->active_button));
-    }
-
-    if (!s->stopped) {
-      s->render->frame_index++;
-      if (s->render->frame_index >= s->n_frames) {
-#if 0
-        SWFDEC_WARNING ("iterating past end");
-        return FALSE;
-#endif
-        s->stopped = TRUE;
-        s->render->frame_index = s->n_frames - 1;
-      }
-    }
+  SWFDEC_DEBUG ("mouse button %d old_mouse_button %d active_button %p",
+      s->mouse_button, s->old_mouse_button, s->render->active_button);
+  if (s->mouse_button && !s->old_mouse_button &&
+      s->render->active_button) {
+              /* FIXME? button down->up/up->down */
+    SWFDEC_DEBUG("executing button");
+    swfdec_button_execute (s, SWFDEC_BUTTON (s->render->active_button));
   }
 
+  for (g=s->execute_list; g; g = g->next) {
+    SwfdecBuffer *buffer = g->data;
+    swfdec_action_script_execute (s, buffer);
+  }
+  g_list_free (s->execute_list);
+  s->execute_list = NULL;
+
   s->render->active_button = NULL;
-
-  for (g = g_list_first (s->render->object_states); g; g = g_list_next (g)) {
-    SwfdecRenderState *state = g->data;
-
-    state->frame_index++;
-    SWFDEC_INFO ("iterate layer=%d frame_index=%d", state->layer,
-        state->frame_index);
-  }
-
   s->old_mouse_button = s->mouse_button;
 
-  return TRUE;
-}
-
-SwfdecRenderState *
-swfdec_render_get_object_state (SwfdecRender * render, int layer, int id)
-{
-  GList *g;
-
-  for (g = g_list_first (render->object_states); g; g = g_list_next (g)) {
-    SwfdecRenderState *state = g->data;
-
-    if (state->layer == layer && state->id == id)
-      return state;
+  if (s->next_frame == -1) {
+    if (!s->main_sprite_seg->stopped) {
+      s->next_frame = s->render->frame_index + 1;
+      if (s->next_frame >= s->n_frames) {
+        if (0 /*s->repeat*/) {
+          s->next_frame = 0;
+        } else {
+          s->next_frame = s->n_frames - 1;
+        }
+      }
+    } else {
+      s->next_frame = s->render->frame_index;
+    }
   }
 
-  return NULL;
-}
-
-void
-swfdec_render_seek (SwfdecDecoder * s, int frame)
-{
-  if (frame < 0 || frame >= s->n_frames)
-    return;
-
-  s->render->seek_frame = frame;
+  return TRUE;
 }
 
 int
@@ -145,6 +117,7 @@ swfdec_render_get_image (SwfdecDecoder * s)
 {
   SwfdecSpriteSegment *seg;
   SwfdecBuffer *buffer;
+  SwfdecSpriteFrame *frame;
   GList *g;
   GList *render_list = NULL;
   int clip_depth = 0;
@@ -155,7 +128,7 @@ swfdec_render_get_image (SwfdecDecoder * s)
 
   g_return_val_if_fail (s->render->frame_index < s->n_frames, NULL);
 
-  SWFDEC_DEBUG ("swf_render_frame");
+  SWFDEC_DEBUG ("swf_render_frame index %d", s->render->frame_index);
 
   s->render->drawrect.x0 = 0;
   s->render->drawrect.x1 = 0;
@@ -166,16 +139,10 @@ swfdec_render_get_image (SwfdecDecoder * s)
   SWFDEC_DEBUG ("inval rect %d %d %d %d", s->render->drawrect.x0,
       s->render->drawrect.x1, s->render->drawrect.y0, s->render->drawrect.y1);
 
-  for (g = g_list_last (s->main_sprite->layers); g; g = g_list_previous (g)) {
+  frame = &s->main_sprite->frames[s->render->frame_index];
+  for (g = g_list_last (frame->segments); g; g = g_list_previous (g)) {
     seg = (SwfdecSpriteSegment *) g->data;
 
-    SWFDEC_LOG ("testing seg %d <= %d < %d",
-        seg->first_frame, s->render->frame_index, seg->last_frame);
-    if (seg->first_frame > s->render->frame_index)
-      continue;
-    if (seg->last_frame <= s->render->frame_index)
-      continue;
-
     /* FIXME need to clip layers instead */
     if (seg->clip_depth) {
       SWFDEC_INFO ("clip_depth=%d", seg->clip_depth);
@@ -222,7 +189,8 @@ swfdec_render_get_image (SwfdecDecoder * s)
     int n;
 
     seg = (SwfdecSpriteSegment *) g->data;
-    n = seg->last_frame - s->render->frame_index - 1;
+    //n = seg->last_frame - s->render->frame_index - 1;
+    n = 1;
     if (n==0) break;
     if (i < s->kept_layers) {
       if (n < n_frames) n_frames = n;
@@ -269,7 +237,7 @@ swfdec_render_get_image (SwfdecDecoder * s)
       for(i=0;i<s->height;i++){
         for(j=0;j<s->width;j++){
           if ((i+j)&1){
-            *(uint32_t *)(s->buffer + i*s->stride + j*4) = 0;
+            *(guint32 *)(s->buffer + i*s->stride + j*4) = 0;
           }
         }
       }
@@ -322,15 +290,12 @@ swfdec_render_get_image (SwfdecDecoder * s)
 SwfdecBuffer *
 swfdec_render_get_audio (SwfdecDecoder * s)
 {
-  SwfdecBuffer *buffer;
-  GList *g;
-
   g_return_val_if_fail (s->render->frame_index < s->n_frames, NULL);
 
   if (s->stream_sound_obj) {
     SwfdecBuffer *chunk;
 
-    chunk = s->main_sprite->sound_chunks[s->render->frame_index];
+    chunk = s->main_sprite->frames[s->render->frame_index].sound_chunk;
     if (chunk) {
       SwfdecSound *sound;
       int n;
@@ -348,10 +313,10 @@ swfdec_render_get_audio (SwfdecDecoder * s)
     }
   }
 
-  if (s->main_sprite->sound_play[s->render->frame_index]) {
+  if (s->main_sprite->frames[s->render->frame_index].sound_play) {
     SwfdecSound *sound;
     SwfdecSoundChunk *chunk =
-      s->main_sprite->sound_play[s->render->frame_index];
+      s->main_sprite->frames[s->render->frame_index].sound_play;
 
     SWFDEC_DEBUG("chunk %p frame_index %d", chunk, s->render->frame_index);
     SWFDEC_DEBUG("play sound object=%d start=%d stop=%d stopflag=%d no_restart=%d loop_count=%d",
@@ -360,40 +325,9 @@ swfdec_render_get_audio (SwfdecDecoder * s)
 
     sound = SWFDEC_SOUND(swfdec_object_get (s, chunk->object));
     if (sound) {
-      int i;
-      GList *g;
-      int n_loops;
-
-      SWFDEC_DEBUG ("sound object n_samples=%d", sound->n_samples);
-
-      for(g=g_list_first(s->stream_sound_buffers);g;g=g_list_next(g)) {
-        swfdec_buffer_unref ((SwfdecBuffer *)g->data);
-      }
-      g_list_free (s->stream_sound_buffers);
-      s->stream_sound_buffers = NULL;
-
-      /* FIXME: loop_count should be handled in some other way */
-      n_loops = chunk->loop_count;
-      if (n_loops > 10) n_loops = 10;
-      for(i=0;i<n_loops;i++){
-        for(g=g_list_first(sound->decoded_buffers);g;g=g_list_next(g)) {
-          SwfdecBuffer *buffer = (SwfdecBuffer *)g->data;
-          swfdec_buffer_ref (buffer);
-          swfdec_decoder_sound_buffer_append (s, buffer);
-        }
-      }
+      swfdec_audio_add_sound (s, sound, chunk->loop_count);
     }
-    
   }
 
-  swfdec_sound_render (s);
-
-  g = g_list_first (s->sound_buffers);
-  if (!g)
-    return NULL;
-
-  buffer = (SwfdecBuffer *) g->data;
-  s->sound_buffers = g_list_delete_link (s->sound_buffers, g);
-
-  return buffer;
+  return swfdec_audio_render (s, 44100/s->rate);
 }
