@@ -13,7 +13,6 @@ G_BEGIN_DECLS
 struct swfdec_render_struct
 {
   int frame_index;
-  int seek_frame;
   SwfdecRect drawrect;
 
   GList *object_states;
@@ -23,13 +22,6 @@ struct swfdec_render_struct
   SwfdecObject *active_button;
 };
 
-struct _SwfdecRenderState
-{
-  int layer;
-  int id;
-  int frame_index;
-};
-
 SwfdecRender *swfdec_render_new (void);
 void swfdec_render_free (SwfdecRender * render);
 gboolean swfdec_render_iterate (SwfdecDecoder * s);
@@ -37,8 +29,6 @@ void swfdec_render_seek (SwfdecDecoder * s, int frame);
 SwfdecBuffer *swfdec_render_get_image (SwfdecDecoder * s);
 SwfdecBuffer *swfdec_render_get_audio (SwfdecDecoder * s);
 int swfdec_render_get_frame_index (SwfdecDecoder * s);
-SwfdecRenderState *swfdec_render_get_object_state (SwfdecRender * render,
-    int layer, int id);
 void swfdec_render_resize (SwfdecDecoder *s);
 
 /* functions implemented by rendering backends */
