@@ -69,6 +69,7 @@ swfdec_shapevec_free (SwfdecShapeVec * shapevec)
   g_free (shapevec);
 }
 
+#if 0
 static int
 get_shape_rec (SwfdecBits * bits, int n_fill_bits, int n_line_bits)
 {
@@ -190,48 +191,7 @@ get_shape_rec (SwfdecBits * bits, int n_fill_bits, int n_line_bits)
 
   return 1;
 }
-
-int
-tag_func_define_shape (SwfdecDecoder * s)
-{
-  SwfdecBits *b = &s->b;
-  int id;
-  int n_fill_styles;
-  int n_line_styles;
-  int n_fill_bits;
-  int n_line_bits;
-  int rect[4];
-  int i;
-
-  id = swfdec_bits_get_u16 (b);
-  SWFDEC_INFO ("id=%d", id);
-  SWFDEC_DEBUG ("  bounds = %s\n", "rect");
-  swfdec_bits_get_rect (b, rect);
-  swfdec_bits_syncbits (b);
-  n_fill_styles = swfdec_bits_get_u8 (b);
-  SWFDEC_LOG ("  n_fill_styles = %d", n_fill_styles);
-  for (i = 0; i < n_fill_styles; i++) {
-    swfdec_bits_get_fill_style (b);
-  }
-  swfdec_bits_syncbits (b);
-  n_line_styles = swfdec_bits_get_u8 (b);
-  SWFDEC_LOG ("  n_line_styles = %d", n_line_styles);
-  for (i = 0; i < n_line_styles; i++) {
-    swfdec_bits_get_line_style (b);
-  }
-  swfdec_bits_syncbits (b);
-  n_fill_bits = swfdec_bits_getbits (b, 4);
-  n_line_bits = swfdec_bits_getbits (b, 4);
-  SWFDEC_LOG ("  n_fill_bits = %d", n_fill_bits);
-  SWFDEC_LOG ("  n_line_bits = %d", n_line_bits);
-  do {
-    SWFDEC_LOG ("  shape_rec:");
-  } while (get_shape_rec (b, n_fill_bits, n_line_bits));
-
-  swfdec_bits_syncbits (b);
-
-  return SWF_OK;
-}
+#endif
 
 SwfdecShapeVec *
 swf_shape_vec_new (void)
@@ -334,6 +294,7 @@ swf_shape_add_styles (SwfdecDecoder * s, SwfdecShape * shape, SwfdecBits * bits)
     fill_style_type = swfdec_bits_get_u8 (bits);
     SWFDEC_LOG ("    type 0x%02x", fill_style_type);
     if (fill_style_type == 0x00) {
+      shapevec->fill_type = fill_style_type;
       if (shape->rgba) {
         shapevec->color = swfdec_bits_get_rgba (bits);
       } else {
@@ -348,11 +309,12 @@ swf_shape_add_styles (SwfdecDecoder * s, SwfdecShape * shape, SwfdecBits * bits)
       } else {
         shapevec->grad = swfdec_bits_get_gradient (bits);
       }
+      swfdec_bits_syncbits (bits);
       shapevec->fill_transform.trans[0] *= SWF_SCALE_FACTOR;
       shapevec->fill_transform.trans[1] *= SWF_SCALE_FACTOR;
       shapevec->fill_transform.trans[2] *= SWF_SCALE_FACTOR;
       shapevec->fill_transform.trans[3] *= SWF_SCALE_FACTOR;
-    } else if (fill_style_type == 0x40 || fill_style_type == 0x41) {
+    } else if (fill_style_type >= 0x40 && fill_style_type <= 0x43) {
       shapevec->fill_type = fill_style_type;
       shapevec->fill_id = swfdec_bits_get_u16 (bits);
       SWFDEC_LOG ("   background fill id = %d (type 0x%02x)",
@@ -364,14 +326,18 @@ swf_shape_add_styles (SwfdecDecoder * s, SwfdecShape * shape, SwfdecBits * bits)
       }
 
       swfdec_bits_get_transform (bits, &shapevec->fill_transform);
+      swfdec_bits_syncbits (bits);
       /* FIXME: the 0.965 is a mysterious factor that seems to improve
        * rendering of images. */
-      shapevec->fill_transform.trans[0] *= SWF_SCALE_FACTOR * 0.965;
-      shapevec->fill_transform.trans[1] *= SWF_SCALE_FACTOR * 0.965;
-      shapevec->fill_transform.trans[2] *= SWF_SCALE_FACTOR * 0.965;
-      shapevec->fill_transform.trans[3] *= SWF_SCALE_FACTOR * 0.965;
+//#define MYSTERIOUS_FACTOR 0.965
+#define MYSTERIOUS_FACTOR 1.0
+      shapevec->fill_transform.trans[0] *= SWF_SCALE_FACTOR * MYSTERIOUS_FACTOR;
+      shapevec->fill_transform.trans[1] *= SWF_SCALE_FACTOR * MYSTERIOUS_FACTOR;
+      shapevec->fill_transform.trans[2] *= SWF_SCALE_FACTOR * MYSTERIOUS_FACTOR;
+      shapevec->fill_transform.trans[3] *= SWF_SCALE_FACTOR * MYSTERIOUS_FACTOR;
     } else {
       SWFDEC_ERROR ("unknown fill style type 0x%02x", fill_style_type);
+      shapevec->fill_type = 0;
     }
   }
 
@@ -501,9 +467,9 @@ swf_shape_get_recs (SwfdecDecoder * s, SwfdecBits * bits, SwfdecShape * shape)
       edge_flag = swfdec_bits_getbits (bits, 1);
 
       if (edge_flag == 0) {
-        int16_t x0, y0;
-        int16_t x1, y1;
-        int16_t x2, y2;
+        int x0, y0;
+        int x1, y1;
+        int x2, y2;
 
         x0 = x;
         y0 = y;
@@ -613,7 +579,6 @@ swfdec_shape_compose (SwfdecDecoder * s, SwfdecLayerVec * layervec,
   layervec->color = SWF_COLOR_COMBINE (255, 0, 0, 255);
 
   image = SWFDEC_IMAGE (image_object);
-  SWFDEC_LOG ("image %p", image->image_data);
 
   SWFDEC_LOG ("%g %g %g %g %g %g",
       shapevec->fill_transform.trans[0],
@@ -637,7 +602,7 @@ swfdec_shape_compose (SwfdecDecoder * s, SwfdecLayerVec * layervec,
   mat0.trans[5] -= layervec->rect.y0;
   swfdec_transform_invert (&mat, &mat0);
   dest = layervec->compose;
-  src = image->image_data;
+  src = swfdec_handle_get_data (image->handle);
   inv_width = 1.0 / image->width;
   inv_height = 1.0 / image->height;
   for (j = 0; j < height; j++) {
@@ -654,14 +619,27 @@ swfdec_shape_compose (SwfdecDecoder * s, SwfdecLayerVec * layervec,
 #else
       ix = x;
       iy = y;
-      if (x < 0)
-        ix = 0;
-      if (x > image->width - 1)
-        ix = image->width - 1;
-      if (y < 0)
-        iy = 0;
-      if (y > image->height - 1)
-        iy = image->height - 1;
+      if (shapevec->fill_type == 0x40 || shapevec->fill_type == 0x42) {
+	/* Repeating image */
+        while (ix < 0)
+	  ix += image->width;
+        if (ix > image->width - 1)
+          ix %= image->width;
+        while (iy < 0)
+          iy += image->height;
+        if (iy > image->height - 1)
+          iy %= image->height;
+      } else {
+	/* Clipped image */
+        if (ix < 0)
+          ix = 0;
+        if (ix > image->width - 1)
+          ix = image->width - 1;
+        if (iy < 0)
+          iy = 0;
+        if (iy > image->height - 1)
+          iy = image->height - 1;
+      }
 
 #endif
 #define RGBA8888_COPY(a,b) (*(guint32 *)(a) = *(guint32 *)(b))
@@ -896,7 +874,7 @@ tag_define_morph_shape (SwfdecDecoder * s)
   return SWF_OK;
 }
 
-void
+static void
 swf_shape_ignore_recs (SwfdecDecoder * s, SwfdecBits * bits,
     SwfdecShape * shape)
 {
@@ -957,9 +935,9 @@ swf_shape_ignore_recs (SwfdecDecoder * s, SwfdecBits * bits,
       edge_flag = swfdec_bits_getbits (bits, 1);
 
       if (edge_flag == 0) {
-        int16_t x0, y0;
-        int16_t x1, y1;
-        int16_t x2, y2;
+        int x0, y0;
+        int x1, y1;
+        int x2, y2;
 
         x0 = x;
         y0 = y;
@@ -1068,6 +1046,7 @@ swf_morphshape_add_styles (SwfdecDecoder * s, SwfdecShape * shape,
     fill_style_type = swfdec_bits_get_u8 (bits);
     SWFDEC_LOG ("    type 0x%02x", fill_style_type);
     if (fill_style_type == 0x00) {
+      shapevec->fill_type = fill_style_type;
       shapevec->color = swfdec_bits_get_rgba (bits);
       end_color = swfdec_bits_get_rgba (bits);
       SWFDEC_LOG ("    color %08x", shapevec->color);
@@ -1101,6 +1080,7 @@ swf_morphshape_add_styles (SwfdecDecoder * s, SwfdecShape * shape,
       shapevec->fill_transform.trans[3] *= SWF_SCALE_FACTOR * 0.965;
     } else {
       SWFDEC_ERROR ("unknown fill style type 0x%02x", fill_style_type);
+      shapevec->fill_type = 0;
     }
   }
 
@@ -1193,9 +1173,9 @@ swf_morphshape_get_recs (SwfdecDecoder * s, SwfdecBits * bits,
       edge_flag = swfdec_bits_getbits (bits, 1);
 
       if (edge_flag == 0) {
-        int16_t x0, y0;
-        int16_t x1, y1;
-        int16_t x2, y2;
+        int x0, y0;
+        int x1, y1;
+        int x2, y2;
 
         x0 = x;
         y0 = y;
