@@ -17,8 +17,6 @@ static void swfdec_sound_mp3_cleanup (SwfdecSound * sound);
 #ifdef HAVE_MAD
 static SwfdecBuffer * convert_synth_to_buffer (SwfdecSound *sound);
 #endif
-void swfdec_decoder_sound_buffer_append (SwfdecDecoder * s,
-    SwfdecBuffer * buffer);
 
 void adpcm_decode (SwfdecDecoder * s, SwfdecSound * sound);
 
@@ -145,12 +143,6 @@ tag_func_define_sound (SwfdecDecoder * s)
   return SWF_OK;
 }
 
-void
-swfdec_decoder_sound_buffer_append (SwfdecDecoder * s, SwfdecBuffer * buffer)
-{
-  s->stream_sound_buffers = g_list_append (s->stream_sound_buffers, buffer);
-}
-
 int
 tag_func_sound_stream_head (SwfdecDecoder * s)
 {
@@ -274,7 +266,7 @@ tag_func_start_sound (SwfdecDecoder * s)
   chunk = get_soundinfo (b);
   chunk->object = id;
 
-  s->parse_sprite->sound_play[s->parse_sprite->parse_frame] = chunk;
+  s->parse_sprite->frames[s->parse_sprite->parse_frame].sound_play = chunk;
 
   return SWF_OK;
 }
@@ -299,12 +291,12 @@ tag_func_define_button_sound (SwfdecDecoder * s)
   return SWF_OK;
 }
 
-int index_adjust[16] = {
+static int index_adjust[16] = {
   -1, -1, -1, -1, 2, 4, 6, 8,
   -1, -1, -1, -1, 2, 4, 6, 8,
 };
 
-int step_size[89] = {
+static int step_size[89] = {
   7, 8, 9, 10, 11, 12, 13, 14, 16, 17, 19, 21, 23, 25, 28, 31,
   34, 37, 41, 45, 50, 55, 60, 66, 73, 80, 88, 97, 107, 118,
   130, 143, 157, 173, 190, 209, 230, 253, 279, 307, 337, 371,
@@ -378,6 +370,7 @@ adpcm_decode (SwfdecDecoder * s, SwfdecSound * sound)
   }
 }
 
+#if 0
 void
 swfdec_sound_render (SwfdecDecoder * s)
 {
@@ -426,6 +419,7 @@ swfdec_sound_render (SwfdecDecoder * s)
 
   s->sound_buffers = g_list_append (s->sound_buffers, buffer);
 }
+#endif
 
 
 #ifdef HAVE_MAD
@@ -513,7 +507,7 @@ swfdec_sound_mp3_decode_stream (SwfdecDecoder * s, SwfdecSound * sound)
 
     buffer = convert_synth_to_buffer (sound);
     if (buffer) {
-      swfdec_decoder_sound_buffer_append (s, buffer);
+      swfdec_audio_stream_push_buffer (s, 0, buffer);
     }
   }
 
@@ -628,7 +622,7 @@ swfdec_sound_mp3_init (SwfdecSound * sound)
 
 }
 
-void
+static void
 swfdec_sound_mp3_decode (SwfdecSound * sound)
 {
 }
