@@ -6,9 +6,11 @@ typedef struct _SwfdecActionContext SwfdecActionContext;
 typedef struct _SwfdecBuffer SwfdecBuffer;
 typedef struct _SwfdecBufferQueue SwfdecBufferQueue;
 typedef struct _SwfdecButton SwfdecButton;
+typedef struct _SwfdecCache SwfdecCache;
 typedef struct _SwfdecColorTransform SwfdecColorTransform;
 typedef struct _SwfdecDecoder SwfdecDecoder;
 typedef struct _SwfdecFont SwfdecFont;
+typedef struct _SwfdecHandle SwfdecHandle;
 typedef struct _SwfdecImage SwfdecImage;
 typedef struct _SwfdecObject SwfdecObject;
 typedef struct _SwfdecShape SwfdecShape;
@@ -19,6 +21,7 @@ typedef struct _SwfdecSound SwfdecSound;
 typedef struct _SwfdecSoundChunk SwfdecSoundChunk;
 typedef struct _SwfdecSprite SwfdecSprite;
 typedef struct _SwfdecSpriteSegment SwfdecSpriteSegment;
+typedef struct _SwfdecSpriteFrame SwfdecSpriteFrame;
 typedef struct _SwfdecText SwfdecText;
 typedef struct _SwfdecTextGlyph SwfdecTextGlyph;
 typedef struct _SwfdecTransform SwfdecTransform;
