@@ -7,5 +7,7 @@
 #define SPP_EOF 2
 #define SPP_SIZE 3
 #define SPP_GO_TO_URL 3
+#define SPP_GO_TO_URL2 4
+#define SPP_METADATA 5
 
 #endif
