@@ -3,9 +3,8 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <glib.h>
-#include <gst/gconf/gconf.h>
-#include <gst/xoverlay/xoverlay.h>
-#include <gst/navigation/navigation.h>
+#include <gst/interfaces/xoverlay.h>
+#include <gst/interfaces/navigation.h>
 
 #include <getopt.h>
 #include <stdlib.h>
@@ -22,6 +21,8 @@
 #include "gstappsrc.h"
 #include "spp.h"
 
+#define MAX_VERSION 100
+
 #define static
 
 struct sound_buffer_struct
@@ -34,13 +35,10 @@ typedef struct sound_buffer_struct SoundBuffer;
 
 gboolean debug = FALSE;
 
-gboolean streaming = FALSE;
 unsigned char *image;
 
 GtkWidget *gtk_wind;
 
-//GtkWidget *video_widget;
-int plugged;
 GdkWindow *gdk_parent;
 
 GIOChannel *input_chan;
@@ -50,8 +48,6 @@ guint render_idle_id;
 
 unsigned long xid;
 unsigned long eb_xid;
-int width;
-int height;
 int fast = FALSE;
 int enable_sound = TRUE;
 int quit_at_eof = FALSE;
@@ -60,31 +56,40 @@ int safe = FALSE;
 int interval;
 struct timeval image_time;
 
-int video_window_x;
-int video_window_y;
-int video_window_w;
-int video_window_h;
-
-GdkWindow *video_window;
+char *filename;
 
 gboolean visible;
 gboolean iconified;
+
+/* state variables */
+gboolean have_pipeline;
+gboolean is_plugged;
+gboolean error_occurred;
+gboolean streaming = FALSE;
 gboolean playing;
 
+/* functions */
 static void do_help (void);
 static void print_formats (void);
-static void new_gtk_window (void);
+static gboolean create_ui (void);
+gboolean ui_create_pipeline(gpointer ignored);
+static gboolean create_pipeline (int streaming, const char *location);
+static void handle_error (const char *format, ...);
+static const char * get_playbin_uri (void);
 
-static void got_source (GObject * playbin, GParamSpec pspec, gpointer closure);
+/* Gstreamer callbacks */
 static void desired_size (GObject * obj, int w, int h, gpointer closure);
-static void embed_url (GObject * obj, const char *url, gpointer closure);
+static void embed_url (GObject * obj, const char *url, const char *target, gpointer closure);
 
 static void packet_write (int fd, int code, int len, const char *s);
+static void packet_go_to_url (const char *url, const char *target);
+static GstBusSyncReply bus_sync_handler (GstBus * bus, GstMessage * message, GstPipeline * pipeline);
+static gboolean bus_async_handler (GstBus * bus, GstMessage * message, gpointer ignored);
 
 /* GTK callbacks */
 static void destroy_cb (GtkWidget * widget, gpointer data);
 static void embedded (GtkPlug * plug, gpointer data);
-static int expose_cb (GtkWidget * widget, GdkEventExpose * evt, gpointer data);
+static int expose (GtkWidget * widget, GdkEventExpose * evt, gpointer data);
 static int button_press_event (GtkWidget * widget, GdkEventButton * evt,
     gpointer data);
 static int button_release_event (GtkWidget * widget, GdkEventButton * evt,
@@ -109,18 +114,12 @@ void fault_restore (void);
 void fault_setup (void);
 
 static GstElement *pipeline;
-static GstElement *thread;
-static GstElement *src;
-static GstElement *swfdec;
-static GstElement *thread2;
-static GstElement *thread3;
-static GstElement *queue1;
-static GstElement *queue2;
-static GstElement *audioconvert;
-static GstElement *audioscale;
-static GstElement *audio_sink;
 static GstElement *video_sink;
 static GstElement *xoverlay;
+static GstElement *playbin;
+
+static GstElement *appsrc;
+static GstElement *fakesink;
 
 
 int
@@ -140,7 +139,6 @@ main (int argc, char *argv[])
     {"print-formats", 0, NULL, 0x1000},
     {0},
   };
-  int ret;
 
 #if 0
   log_fd = g_file_open_tmp ("gst-mozilla-plugin.log.XXXXXX", NULL, NULL);
@@ -187,136 +185,15 @@ main (int argc, char *argv[])
     }
   }
 
-#if 0
-  if (optind != argc - 1)
-    do_help ();
-
-  if (strcmp (argv[optind], "-") == 0) {
-    streaming = TRUE;
-  } else {
-    streaming = FALSE;
-  }
-#endif
-
-  pipeline = GST_ELEMENT (gst_pipeline_new (NULL));
-  if (pipeline == NULL) {
-    fprintf(stderr, "Could not create pipeline element\n");
-    exit(1);
-  }
-
-  thread = GST_ELEMENT (gst_thread_new (NULL));
-  if (thread == NULL) {
-    fprintf(stderr, "Could not create thread element\n");
-    exit(1);
-  }
-
-  if (streaming) {
-    src = gst_element_factory_make ("appsrc", NULL);
-    if (src == NULL) {
-      fprintf(stderr, "Could not create source element (appsrc)\n");
-      exit(1);
-    }
-  } else {
-    src = gst_element_factory_make ("filesrc", NULL);
-    if (src == NULL) {
-      fprintf(stderr, "Could not create source element (filesrc)\n");
-      exit(1);
-    }
-    g_object_set (src, "location", argv[optind], NULL);
-  }
-
-  swfdec = gst_element_factory_make ("swfdec2", NULL);
-  if (swfdec == NULL) {
-    fprintf(stderr, "Could not create swfdec element\n");
-    exit(1);
-  }
-
-  thread2 = gst_element_factory_make ("thread", NULL);
-  if (thread2 == NULL) {
-    fprintf(stderr, "Could not create thread2 element\n");
-    exit(1);
-  }
-
-  queue1 = gst_element_factory_make ("queue", NULL);
-  if (queue1 == NULL) {
-    fprintf(stderr, "Could not create queue element\n");
-    exit(1);
-  }
-
-  thread3 = gst_element_factory_make ("thread", NULL);
-  if (thread3 == NULL) {
-    fprintf(stderr, "Could not create thread2 element\n");
-    exit(1);
-  }
-
-  queue2 = gst_element_factory_make ("queue", NULL);
-  if (queue2 == NULL) {
-    fprintf(stderr, "Could not create queue element\n");
-    exit(1);
-  }
-  g_object_set (G_OBJECT(queue2), "max-size-buffers", 1, NULL);
-
-  audioconvert = gst_element_factory_make ("audioconvert", NULL);
-  if (audioconvert == NULL) {
-    fprintf(stderr, "Could not create audioconvert element\n");
-    exit(1);
-  }
-
-  audioscale = gst_element_factory_make ("audioscale", NULL);
-  if (audioscale == NULL) {
-    fprintf(stderr, "Could not create audioscale element\n");
-    exit(1);
-  }
-
-  audio_sink = gst_gconf_get_default_audio_sink ();
-  if (audio_sink == NULL) {
-    fprintf(stderr, "Could not create audio sink\n");
-    exit(1);
-  }
-
-  video_sink = gst_element_factory_make ("ximagesink", NULL);
-  if (video_sink == NULL) {
-    fprintf(stderr, "Could not create video sink (ximagesink)\n");
-    exit(1);
-  }
-  xoverlay = video_sink;
-
-  g_signal_connect (G_OBJECT (xoverlay), "desired-size-changed",
-      G_CALLBACK (desired_size), NULL);
-
-  g_signal_connect (G_OBJECT (swfdec), "embed-url",
-      G_CALLBACK (embed_url), NULL);
-
-  gst_bin_add (GST_BIN (pipeline), thread);
-  gst_bin_add_many (GST_BIN (thread), src, swfdec, thread2, thread3, NULL);
-  gst_bin_add_many (GST_BIN (thread2), queue1, audioconvert, audioscale,
-      audio_sink, NULL);
-  gst_bin_add_many (GST_BIN (thread3), queue2, video_sink, NULL);
-
-  ret = gst_element_link (src, swfdec);
-  ret &= gst_element_link (swfdec, queue2);
-  ret &= gst_element_link (queue2, video_sink);
-  ret &= gst_element_link (swfdec, queue1);
-  ret &= gst_element_link (queue1, audioscale);
-  ret &= gst_element_link (audioscale, audioconvert);
-  ret &= gst_element_link (audioconvert, audio_sink);
-  if (ret == FALSE) {
-    fprintf(stderr, "Failed to link elements\n");
-    exit(1);
-  }
-
   if (xid) {
-    plugged = 1;
+    is_plugged = 1;
   }
 
-  new_gtk_window ();
+  create_ui ();
 
-  playing = TRUE;
-  ret = gst_element_set_state (GST_ELEMENT (pipeline), GST_STATE_PLAYING);
-  if (ret != GST_STATE_SUCCESS) {
-    fprintf(stderr, "Failed to set pipeline to PLAYING\n");
-    exit(1);
-  }
+  filename = argv[optind];
+
+  g_idle_add (ui_create_pipeline, NULL);
 
   g_timeout_add (1000, timeout, NULL);
 
@@ -325,6 +202,114 @@ main (int argc, char *argv[])
   exit (0);
 }
 
+gboolean
+ui_create_pipeline(gpointer ignored)
+{
+  int ret;
+
+  if (filename || streaming) {
+    have_pipeline = create_pipeline(streaming, filename);
+  } else {
+    have_pipeline = FALSE;
+  }
+
+  ret = gst_element_set_state (GST_ELEMENT (pipeline), GST_STATE_PLAYING);
+  if (ret == GST_STATE_CHANGE_FAILURE) {
+    handle_error ("GStreamer error: Failed to set pipeline to PLAYING.");
+  }
+  if (debug)
+    fprintf(stderr, "state change %d\n", ret);
+  playing = TRUE;
+
+  if (safe) {
+    handle_error ("The SWF file caused a fatal error in the swfdec decoder. "
+      " This likely means that there is a bug in swfdec.");
+  }
+  
+  return FALSE;
+}
+
+#define CREATE_ELEMENT(e,type) \
+  do { \
+    e = gst_element_factory_make (type, NULL); \
+    if (e == NULL) { \
+      char *s; \
+      if (bad_elements) { \
+        s = g_strconcat (bad_elements, " ", type, NULL); \
+        g_free(bad_elements); \
+        bad_elements = s; \
+      } else { \
+        bad_elements = g_strdup (type); \
+      } \
+    } \
+  } while (0)
+
+
+static gboolean
+create_pipeline (int streaming, const char *location)
+{
+  GstBus *bus;
+  char *bad_elements = NULL;
+  char *uri;
+  
+  CREATE_ELEMENT (pipeline, "pipeline");
+  if (streaming) {
+    uri = g_strdup_printf("appsrc://");
+  } else {
+    if (location == NULL) {
+      handle_error ("No file specified on command line and not in streaming mode.");
+      return FALSE;
+    }
+    uri = g_strdup_printf("file://%s", location);
+  }
+  if (safe) {
+    CREATE_ELEMENT (appsrc, "appsrc");
+    CREATE_ELEMENT (fakesink, "fakesink");
+  } else {
+    CREATE_ELEMENT (video_sink, "ximagesink");
+    CREATE_ELEMENT (playbin, "playbin");
+  }
+
+  if (bad_elements) {
+    handle_error ("The following GStreamer elements are required, but are missing: %s",
+        bad_elements);
+    g_free(bad_elements);
+    return FALSE;
+  }
+
+  if (safe) {
+    int ret;
+
+    gst_bin_add_many (GST_BIN (pipeline), appsrc, fakesink, NULL);
+    ret = gst_element_link (appsrc, fakesink);
+    if (!ret) {
+      handle_error("GStreamer error: could not link appsrc and fakesink elements");
+    }
+
+  } else {
+    g_object_set (playbin,
+        "video-sink", video_sink,
+        "uri", uri,
+        "queue-size", (guint64)(GST_MSECOND * 1),
+        //"queue-threshold", (guint64)(GST_MSECOND * 10),
+        NULL);
+
+    xoverlay = video_sink;
+
+    /* FIXME */
+    //g_signal_connect (G_OBJECT (xoverlay), "desired-size-changed", G_CALLBACK (desired_size), NULL);
+
+    gst_bin_add_many (GST_BIN (pipeline), playbin, NULL);
+
+    bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
+    gst_bus_set_sync_handler (bus, (GstBusSyncHandler) bus_sync_handler, pipeline);
+
+    gst_bus_add_watch (bus, (GstBusFunc) bus_async_handler, NULL);
+  }
+
+  return TRUE;
+}
+
 static void
 do_help (void)
 {
@@ -339,10 +324,13 @@ print_formats (void)
   exit (0);
 }
 
-static void
-new_gtk_window (void)
+static gboolean
+create_ui (void)
 {
-  if (plugged) {
+  int width = 320;
+  int height = 240;
+
+  if (is_plugged) {
     gtk_wind = gtk_plug_new (0);
     gtk_signal_connect (GTK_OBJECT (gtk_wind), "embedded",
         GTK_SIGNAL_FUNC (embedded), NULL);
@@ -352,6 +340,7 @@ new_gtk_window (void)
   } else {
     gtk_wind = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   }
+  //g_object_set (G_OBJECT(gtk_wind), "visible", FALSE, NULL);
   gtk_window_set_default_size (GTK_WINDOW (gtk_wind), width, height);
   gtk_signal_connect (GTK_OBJECT (gtk_wind), "delete_event",
       GTK_SIGNAL_FUNC (destroy_cb), NULL);
@@ -378,6 +367,10 @@ new_gtk_window (void)
       GTK_SIGNAL_FUNC (button_press_event), NULL);
   g_signal_connect (G_OBJECT (gtk_wind), "button_release_event",
       GTK_SIGNAL_FUNC (button_release_event), NULL);
+#if 0
+  g_signal_connect (G_OBJECT (gtk_wind), "expose_event",
+      GTK_SIGNAL_FUNC (expose), NULL);
+#endif
 
   gtk_widget_add_events (gtk_wind,
       GDK_LEAVE_NOTIFY_MASK
@@ -388,12 +381,44 @@ new_gtk_window (void)
 
   gtk_widget_show_all (gtk_wind);
 
-  if (plugged) {
+  if (is_plugged) {
     XReparentWindow (GDK_WINDOW_XDISPLAY (gtk_wind->window),
         GDK_WINDOW_XID (gtk_wind->window), xid, 0, 0);
     XMapWindow (GDK_WINDOW_XDISPLAY (gtk_wind->window),
         GDK_WINDOW_XID (gtk_wind->window));
   }
+
+  return TRUE;
+}
+
+static void
+handle_error (const char *format, ...)
+{
+  GtkWidget *label;
+  GtkWidget *align;
+  char *error_message;
+  va_list varargs;
+
+  va_start (varargs, format);
+  error_message = g_strdup_vprintf (format, varargs);
+  va_end (varargs);
+
+#if 0
+  if (playing) {
+    gst_element_set_state (GST_ELEMENT (pipeline), GST_STATE_PAUSED);
+  }
+#endif
+
+  label = gtk_label_new(error_message);
+  g_free(error_message);
+  
+  align = gtk_alignment_new (0.5, 0.5, 0, 0);
+  gtk_container_add (GTK_CONTAINER(gtk_wind), align);
+  gtk_container_add (GTK_CONTAINER(align), label);
+  gtk_widget_show_all (align);
+
+  g_signal_connect (G_OBJECT (align), "expose_event",
+      GTK_SIGNAL_FUNC (expose), NULL);
 }
 
 /* GTK callbacks */
@@ -401,17 +426,51 @@ new_gtk_window (void)
 static gboolean
 timeout (gpointer closure)
 {
-  if (debug) {
-    if (GTK_WIDGET_MAPPED(gtk_wind)){
-      fprintf(stderr, "is mapped\n");
-    }else{
-      fprintf(stderr, "is unmapped\n");
-    }
-  }
 
   return TRUE;
 }
 
+static void
+menu_open (GtkMenuItem *item, gpointer user_data)
+{
+  const char *url = get_playbin_uri();
+
+  if (url) {
+    packet_go_to_url(url, "_top");
+  }
+}
+
+static void
+menu_report_bug (GtkMenuItem *item, gpointer user_data)
+{
+  char *s;
+
+  if (streaming) {
+    const char *url = get_playbin_uri();
+
+    if (url) {
+      s = g_strdup_printf("http://www.schleef.org/swfdec/bugreport.html?%s", url);
+      packet_go_to_url(s, "_top");
+      g_free(s);
+    }
+  }
+}
+
+static void
+menu_copy_url (GtkMenuItem *item, gpointer user_data)
+{
+  const char *url = get_playbin_uri();
+  GtkClipboard *clipboard;
+
+  if (url) {
+    clipboard = gtk_clipboard_get_for_display (gdk_display_get_default(),
+        GDK_SELECTION_PRIMARY);
+
+    gtk_clipboard_set_text (clipboard, url, strlen(url));
+  }
+}
+
+
 static int
 button_press_event (GtkWidget * widget, GdkEventButton * evt, gpointer data)
 {
@@ -424,18 +483,24 @@ button_press_event (GtkWidget * widget, GdkEventButton * evt, gpointer data)
     GtkWidget *item;
 
     menu = gtk_menu_new ();
-    item = gtk_menu_item_new_with_label ("Moo");
+    item = gtk_menu_item_new_with_label ("Open in new page");
+    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_open), NULL);
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-    item = gtk_menu_item_new_with_label ("File bug...");
+    item = gtk_menu_item_new_with_label ("Report bug...");
+    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_report_bug), NULL);
+    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+    item = gtk_menu_item_new_with_label ("Copy URL");
+    g_signal_connect (G_OBJECT(item), "activate", G_CALLBACK(menu_copy_url), NULL);
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
     gtk_widget_show_all (menu);
 
     gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, evt->time);
-
   }
   if (evt->button == 1) {
-    gst_navigation_send_mouse_event (GST_NAVIGATION (xoverlay),
-        "mouse-button-press", 0, evt->x, evt->y);
+    if (xoverlay) {
+      gst_navigation_send_mouse_event (GST_NAVIGATION (xoverlay),
+          "mouse-button-press", 0, evt->x, evt->y);
+    }
   }
 
   return FALSE;
@@ -448,8 +513,10 @@ button_release_event (GtkWidget * widget, GdkEventButton * evt, gpointer data)
     fprintf (stderr, "button release\n");
 
   if (evt->button == 1) {
-    gst_navigation_send_mouse_event (GST_NAVIGATION (xoverlay),
-        "mouse-button-release", 0, evt->x, evt->y);
+    if (xoverlay) {
+      gst_navigation_send_mouse_event (GST_NAVIGATION (xoverlay),
+          "mouse-button-release", 0, evt->x, evt->y);
+    }
   }
 
   return FALSE;
@@ -461,8 +528,6 @@ key_press (GtkWidget * widget, GdkEventKey * evt, gpointer data)
   if (debug)
     fprintf (stderr, "key press\n");
 
-  //gtk_exit(0);
-
   return FALSE;
 }
 
@@ -472,19 +537,6 @@ motion_notify (GtkWidget * widget, GdkEventMotion * evt, gpointer data)
   if (debug)
     fprintf (stderr, "motion notify\n");
 
-  /* ximagesink gets these automatically */
-#if 0
-  {
-    int x, y;
-    GdkModifierType state;
-
-    gdk_window_get_pointer (evt->window, &x, &y, &state);
-
-    gst_navigation_send_mouse_event (GST_NAVIGATION (xoverlay),
-        "mouse-move", 0, x, y);
-  }
-#endif
-
   return TRUE;
 }
 
@@ -492,7 +544,7 @@ static void
 check_playing (void)
 {
   gboolean should_play;
-  int ret;
+  //int ret;
 
   if (visible && !iconified) {
     should_play = TRUE;
@@ -500,6 +552,7 @@ check_playing (void)
     should_play = FALSE;
   }
 
+#if 0
 should_play = TRUE;
   if (should_play != playing) {
     if (should_play) {
@@ -515,6 +568,7 @@ should_play = TRUE;
     }
     playing = should_play;
   }
+#endif
 
 }
 
@@ -564,15 +618,17 @@ unmap (GtkWidget *widget, gpointer closure)
 static int
 configure_cb (GtkWidget * widget, GdkEventConfigure * evt, gpointer data)
 {
+  if (debug)
+    fprintf(stderr, "configure\n");
+
   return FALSE;
 }
 
 static int
-expose_cb (GtkWidget * widget, GdkEventExpose * evt, gpointer data)
+expose (GtkWidget * widget, GdkEventExpose * evt, gpointer data)
 {
-  if (image) {
-    gdk_draw_rgb_image (widget->window, widget->style->black_gc,
-        0, 0, width, height, GDK_RGB_DITHER_NONE, image, width * 3);
+  if (xoverlay) {
+    gst_x_overlay_expose (GST_X_OVERLAY(xoverlay));
   }
 
   return FALSE;
@@ -594,18 +650,6 @@ static void
 video_widget_realize (GtkWidget * widget, gpointer ignored)
 {
 
-  video_window = widget->window;
-
-  if (xoverlay && GST_IS_X_OVERLAY (xoverlay) && GDK_IS_WINDOW (video_window)) {
-    /* This is necessary because we're sending an XID for a window that
-     * the X server doesn't necessarily know about yet. */
-    XSync (GDK_DISPLAY (), False);
-    /* Now it knows. */
-    gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (xoverlay),
-        GDK_WINDOW_XID (video_window));
-  } else {
-    g_warning ("Could not find an XOVERLAY element for reparenting");
-  }
 }
 
 static void
@@ -614,12 +658,12 @@ video_widget_allocate (GtkWidget * widget,
 {
   gint x, y, w, h;
 
-  gdk_window_get_geometry (video_window, &x, &y, &w, &h, NULL);
+  gdk_window_get_geometry (gtk_wind->window, &x, &y, &w, &h, NULL);
 
-  video_window_x = x;
-  video_window_y = y;
-  video_window_w = w;
-  video_window_h = h;
+  //video_window_x = x;
+  //video_window_y = y;
+  //video_window_w = w;
+  //video_window_h = h;
 }
 
 static void
@@ -632,13 +676,30 @@ desired_size (GObject * obj, int w, int h, gpointer closure)
 }
 
 static void
-embed_url (GObject * obj, const char *url, gpointer closure)
+embed_url (GObject * obj, const char *url, const char *target, gpointer closure)
 {
+  packet_go_to_url (url, target);
+}
+
+static void
+packet_go_to_url (const char *url, const char *target)
+{
+  char *buf;
+  int len;
+
+  len = strlen (url) + 1 + strlen (target) + 1;
+  buf = g_malloc (len);
+
+  memcpy (buf, url, strlen(url) + 1);
+  memcpy (buf + strlen(url) + 1, target, strlen (target) + 1);
+
   if (streaming) {
-    packet_write (1, SPP_GO_TO_URL, strlen (url), url);
+    packet_write (1, SPP_GO_TO_URL2, len, buf);
   } else {
-    /* FIXME: start a browser? */
+    fprintf(stderr, "go to URL %s\n", url);
   }
+
+  g_free (buf);
 }
 
 static void
@@ -654,3 +715,99 @@ packet_write (int fd, int code, int len, const char *s)
 
   g_free (buf);
 }
+
+static GstBusSyncReply
+bus_sync_handler (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
+{
+  const GstStructure *s;
+  GstMessageType type;
+
+  type = GST_MESSAGE_TYPE (message);
+  s = gst_message_get_structure (message);
+
+  switch (type) {
+    case GST_MESSAGE_ELEMENT:
+      if (gst_structure_has_name (s, "prepare-xwindow-id")) {
+        gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (GST_MESSAGE_SRC (message)),
+            GDK_WINDOW_XID (gtk_wind->window));
+
+        return GST_BUS_DROP;
+      } 
+#if 0
+      if (gst_structure_has_name (s, "embedded-url")) {
+        const char *url;
+        const char *target;
+
+        url = gst_structure_get_string (s, "url");
+        target = gst_structure_get_string (s, "target");
+
+        packet_go_to_url (url, target);
+
+        return GST_BUS_DROP;
+      }
+#endif
+      break;
+    default:
+      break;
+  }
+
+  return GST_BUS_PASS;
+}
+
+static gboolean
+bus_async_handler (GstBus * bus, GstMessage * message, gpointer ignored)
+{
+  const GstStructure *s;
+  GstMessageType type;
+
+  type = GST_MESSAGE_TYPE (message);
+  s = gst_message_get_structure (message);
+
+#if 0
+  fprintf(stderr, "async message type %s\n", gst_structure_to_string(s));
+  fprintf(stderr, "async message type %d\n", type);
+#endif
+
+  switch (type) {
+    case GST_MESSAGE_ELEMENT:
+      if (gst_structure_has_name (s, "embedded-url")) {
+        const char *url;
+        const char *target;
+
+        url = gst_structure_get_string (s, "url");
+        target = gst_structure_get_string (s, "target");
+
+        packet_go_to_url (url, target);
+
+        return GST_BUS_DROP;
+      }
+      break;
+    case GST_MESSAGE_ERROR:
+      gst_element_set_state (GST_ELEMENT (pipeline), GST_STATE_NULL);
+      handle_error("This SWF file is known to trigger bugs in the swfdec decoder.  Playback is cancelled.");
+      break;
+    default:
+      break;
+  }
+
+  return GST_BUS_PASS;
+}
+
+static const char *
+get_playbin_uri (void)
+{
+  char *s = NULL;
+
+  if (appsrc == NULL) {
+    g_object_get(playbin, "source", &appsrc, NULL);
+    if (appsrc == NULL || !GST_IS_APPSRC(appsrc)) {
+      fprintf(stderr, "failed to find appsrc inside playbin\n");
+      return NULL;
+    }
+  }
+
+  g_object_get (appsrc, "source_url", &s, NULL);
+
+  return s;
+}
+
